@@ -135,6 +135,22 @@ PlTerm::get_wchars(unsigned int flags) const
   return L"<---get_wchars error --->"; // Should never get here
 }
 
+_SWI_CPP2_CPP_inline
+const std::string
+PlTerm::get_file_name(int flags) const
+{ char *name = const_cast<char*>("");
+  PlCheckFail(get_file_name(&name, flags));
+  return name;
+}
+
+_SWI_CPP2_CPP_inline
+const std::wstring
+PlTerm::get_file_nameW(int flags) const
+{ wchar_t *name = const_cast<wchar_t*>(L"");
+  PlCheckFail(get_file_nameW(&name, flags));
+  return name;
+}
+
 #define _MUST_BE_TYPE(must_be_name, is_test, type_name)  \
 _SWI_CPP2_CPP_inline \
 void \
@@ -615,7 +631,8 @@ bool
 PlTerm::eq(const char *s) const
 { char *s0;
 
-  if ( get_chars(&s0, CVT_ALL) )
+  PlStringBuffers _string_buffers; 
+  if ( _get_chars(&s0, CVT_ALL) )
     return strcmp(s0, s) == 0;
 
   throw PlTypeError("text", *this);
@@ -626,7 +643,8 @@ bool
 PlTerm::eq(const wchar_t *s) const
 { wchar_t *s0;
 
-  if ( Plx_get_wchars(unwrap(), nullptr, &s0, CVT_ALL) )
+  PlStringBuffers _string_buffers;
+  if ( _get_wchars(nullptr, &s0, CVT_ALL) )
     return wcscmp(s0, s) == 0;
 
   throw PlTypeError("text", *this);
@@ -637,9 +655,10 @@ bool
 PlTerm::eq(const std::string& s) const
 { char *s0;
 
+  PlStringBuffers _string_buffers;
   // Doesn't handle non-NUL terminated - but it's only used by deprecated operator ==
-  if ( get_chars(&s0, CVT_ALL) )
-    return s.compare(s0) == 0; 
+  if ( _get_chars(&s0, CVT_ALL) )
+    return s.compare(s0) == 0;
 
   throw PlTypeError("text", *this);
 }
@@ -747,6 +766,9 @@ PlTermv::PlTermv(PlTerm m0, PlTerm m1)
   : size_(2),
     a0_(Plx_new_term_refs(2))
 { PlEx<bool>(a0_ != (term_t)0);
+  // Commented out code is possibly less efficient:
+  // PlTerm(a0_+0).put_term(m0); // (*this)[0].put_term(m0) does an unnecessary range check
+  // PlTerm(a0_+1).put_term(m1);
   Plx_put_term(a0_+0, m0.unwrap());
   Plx_put_term(a0_+1, m1.unwrap());
 }
@@ -806,8 +828,8 @@ PlException::string_term() const
 { PlFrame fr;
   // Note that the result is a *term*, so it's unencoded (wchar_t
   // or equivalent) and will be encoded when it's output.
-// TODO: remove USE_PRINT_MESSAGE code (obsolete)
-//       - or use with_output_to(string(String), print_message(error, ...))
+  // TODO: remove USE_PRINT_MESSAGE code (obsolete)
+  //       - or use with_output_to(string(String), print_message(error, ...))
 #ifdef USE_PRINT_MESSAGE
   PlTermv av(2);
   PlCheckFail(av[0].unify_term(PlCompound("print_message",
@@ -851,11 +873,9 @@ PlException::what() const throw()
 
 _SWI_CPP2_CPP_inline
 void
-PlException::set_what_str()
+PlException::set_what_str(PlEncoding enc)
 { if ( what_str_.empty() )
-  { // Doing a query inside a query is ... problematic:
-    // TODO: const_cast<PlException*>(this)->what_str_ = string_term().as_string();
-    what_str_ = term().as_string();
+  { what_str_ = term().as_string(enc);
   }
 }
 
@@ -893,6 +913,33 @@ PlQuery::next_solution()
   return rval;
 }
 
+_SWI_CPP2_CPP_inline
+PlEngineCleanupFailed::PlEngineCleanupFailed(int status_and_flags, int rc)
+  : status_and_flags_(status_and_flags),
+    rc_(rc),
+    what_str_("PlEngineCleanupFailed(" +
+              std::to_string(status_and_flags_) + "):")
+{ // TODO: it would be better to build the string lazily in what(),
+  //       what_str_.empty(); but what() is a const method, so
+  //       requires ugly casts.
+  switch( rc_ ) // See comment in ~PlEngine()
+  { case PL_CLEANUP_SUCCESS:
+      what_str_ += "success";
+      break;
+    case PL_CLEANUP_CANCELED:
+      what_str_ += "canceled";
+      break;
+    case PL_CLEANUP_FAILED:
+      what_str_ += "failed";
+      break;
+    case PL_CLEANUP_RECURSIVE:
+      what_str_ += "recursive";
+      break;
+    default: // shouldn't happen
+      what_str_ += "rc=" + std::to_string(rc_);
+  }
+}
+
 
 		 /*******************************
 		 *	    DEBUG		*
