@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2020, University of Amsterdam
+    Copyright (c)  2002-2025, University of Amsterdam
                               VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -48,7 +49,7 @@
             http_status_reply/5,        % +Status, +Out, +HdrExtra,
                                         % +Context, -Code
 
-            http_timestamp/2,           % +Time, -HTTP string
+            http_timestamp/2,           % ?Time, ?HTTPstring
 
             http_post_data/3,           % +Stream, +Data, +HdrExtra
 
@@ -93,6 +94,7 @@
 	    [ integer/3, atom/3, whites/2, blanks_to_nl/2, string/3,
 	      number/3, blanks/2, float/3, nonblanks/3, eos/2
 	    ]).
+:- autoload(library(date), [parse_time/3]).
 :- use_module(library(settings),[setting/4,setting/2]).
 
 :- multifile
@@ -263,7 +265,8 @@ map_exception_to_http_status(_E, _Status, _NewHdr, _NewContext) :-
 :- endif.
 
 :- meta_predicate
-    if_no_head(0, +).
+    if_no_head(0, +),
+    with_encoding(+, +, 0).
 
 %!  http_reply_data(+Data, +Out, +HdrExtra, +Method, -Code) is semidet.
 %
@@ -280,7 +283,7 @@ http_reply_data_(html(HTML), Out, HdrExtra, Method, Code) :-
     !,
     phrase(reply_header(html(HTML), HdrExtra, Code), Header),
     send_reply_header(Out, Header),
-    if_no_head(print_html(Out, HTML), Method).
+    if_no_head(with_encoding(Out, utf8, print_html(Out, HTML)), Method).
 http_reply_data_(file(Type, File), Out, HdrExtra, Method, Code) :-
     !,
     phrase(reply_header(file(Type, File), HdrExtra, Code), Header),
@@ -320,6 +323,16 @@ if_no_head(_, head) :-
 if_no_head(Goal, _) :-
     call(Goal).
 
+with_encoding(Out, Encoding, Goal) :-
+    stream_property(Out, encoding(Old)),
+    (   Old == Encoding
+    ->  call(Goal)
+    ;   setup_call_cleanup(
+            set_stream(Out, encoding(Encoding)),
+            call(Goal),
+            set_stream(Out, encoding(Old)))
+    ).
+
 reply_file(Out, _File, Header, head) :-
     !,
     send_reply_header(Out, Header).
@@ -482,7 +495,7 @@ status_reply(Status, Out, Options) :-
 %!  status_has_content(+StatusTerm, -HTTPCode)
 %
 %   True when StatusTerm  is  a  status   that  usually  comes  with  an
-%   expanatory content message.
+%   explanatory content message.
 
 status_has_content(created(_Location)).
 status_has_content(moved(_To)).
@@ -776,24 +789,33 @@ http_join_headers([H|T], Hdr0, [H|Hdr]) :-
 %   Alternatively we could dynamically choose for ASCII, ISO-Latin-1
 %   or UTF-8.
 
-http_update_encoding(Header0, utf8, [content_type(Type)|Header]) :-
-    select(content_type(Type0), Header0, Header),
+http_update_encoding(Header0, Encoding, Header) :-
+    memberchk(content_type(Type), Header0),
+    !,
+    http_update_encoding(Type, Header0, Encoding, Header).
+http_update_encoding(Header, octet, Header).
+
+http_update_encoding('text/event-stream', Header, utf8, Header) :-
+    !.
+http_update_encoding(Type0, Header0, utf8, [content_type(Type)|Header]) :-
     sub_atom(Type0, 0, _, _, 'text/'),
     !,
+    select(content_type(_), Header0, Header),
+    !,
     (   sub_atom(Type0, S, _, _, ';')
     ->  sub_atom(Type0, 0, S, _, B)
     ;   B = Type0
     ),
     atom_concat(B, '; charset=UTF-8', Type).
-http_update_encoding(Header, Encoding, Header) :-
-    memberchk(content_type(Type), Header),
+http_update_encoding(Type, Header, Encoding, Header) :-
     (   sub_atom_icasechk(Type, _, 'utf-8')
     ->  Encoding = utf8
     ;   http:mime_type_encoding(Type, Encoding)
     ->  true
     ;   mime_type_encoding(Type, Encoding)
+    ->  true
+    ;   Encoding = octet
     ).
-http_update_encoding(Header, octet, Header).
 
 %!  mime_type_encoding(+MimeType, -Encoding) is semidet.
 %
@@ -860,7 +882,7 @@ connection(Header, Close) :-
 %
 %   Decide on the transfer encoding  from   the  Request and the CGI
 %   header.    The    behaviour    depends      on    the    setting
-%   http:chunked_transfer. If =never=, even   explitic  requests are
+%   http:chunked_transfer. If =never=, even   explicit  requests are
 %   ignored. If =on_request=, chunked encoding  is used if requested
 %   through  the  CGI  header  and  allowed    by   the  client.  If
 %   =if_possible=, chunked encoding is  used   whenever  the  client
@@ -897,6 +919,9 @@ http_update_transfer(if_possible, Request, CgiHeader, Transfer, Header) :-
     Transfer \== none,
     !,
     Header = [transfer_encoding(Transfer)|CgiHeader].
+http_update_transfer(_, _, CgiHeader, event_stream, CgiHeader) :-
+    memberchk(content_type('text/event-stream'), CgiHeader),
+    !.
 http_update_transfer(_, _, CgiHeader, none, CgiHeader).
 
 join_transfer(chunked, chunked, chunked) :- !.
@@ -919,7 +944,7 @@ transfer(Header, Transfer) :-
 
 %!  content_length_in_encoding(+Encoding, +In, -Bytes)
 %
-%   Determine hom many bytes are required to represent the data from
+%   Determine how many bytes are required to represent the data from
 %   stream In using the given encoding.  Fails if the data cannot be
 %   represented with the given encoding.
 
@@ -1001,6 +1026,11 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %     * atom(+Type, +Atom)
 %     Send Atom using the indicated MIME-type.
 %
+%     * string(+String)
+%     * string(+Type, +String)
+%     Similar to atom(Atom) and atom(Type,Atom), accepting a SWI-Prolog
+%     string.
+%
 %     * cgi_stream(+Stream, +Len) Read the input from Stream which,
 %     like CGI data starts with a partial HTTP header. The fields of
 %     this header are merged with the provided HdrExtra fields. The
@@ -1308,8 +1338,8 @@ reply_header(html(Tokens), HdrExtra, Code) -->
     vstatus(ok, Code, HdrExtra),
     date(now),
     header_fields(HdrExtra, CLen),
-    content_length(html(Tokens), CLen),
-    content_type(text/html),
+    content_length(html(Tokens, utf8), CLen),
+    content_type(text/html, utf8),
     "\r\n".
 reply_header(file(Type, File), HdrExtra, Code) -->
     vstatus(ok, Code, HdrExtra),
@@ -1349,6 +1379,11 @@ reply_header(cgi_data(Size), HdrExtra, Code) -->
     header_fields(HdrExtra, CLen),
     content_length(Size, CLen),
     "\r\n".
+reply_header(event_stream, HdrExtra, Code) -->
+    vstatus(ok, Code, HdrExtra),
+    date(now),
+    header_fields(HdrExtra, _),
+    "\r\n".
 reply_header(chunked_data, HdrExtra, Code) -->
     vstatus(ok, Code, HdrExtra),
     date(now),
@@ -1644,50 +1679,38 @@ content_length(Reply, Len) -->
     "Content-Length: ", integer(Len),
     "\r\n".
 
+:- meta_predicate
+    print_length(0, -, +, -).
 
-length_of(_, Len) :-
-    nonvar(Len),
-    !.
-length_of(string(String, Encoding), Len) :-
+:- det(length_of/2).
+length_of(_, Len), integer(Len) => true.
+length_of(string(String, Encoding), Len) =>
     length_of(codes(String, Encoding), Len).
-length_of(codes(String, Encoding), Len) :-
-    !,
-    setup_call_cleanup(
-        open_null_stream(Out),
-        ( set_stream(Out, encoding(Encoding)),
-          format(Out, '~s', [String]),
-          byte_count(Out, Len)
-        ),
-        close(Out)).
-length_of(atom(Atom, Encoding), Len) :-
-    !,
-    setup_call_cleanup(
-        open_null_stream(Out),
-        ( set_stream(Out, encoding(Encoding)),
-          format(Out, '~a', [Atom]),
-          byte_count(Out, Len)
-        ),
-        close(Out)).
-length_of(file(File), Len) :-
-    !,
+length_of(codes(String, Encoding), Len) =>
+    print_length(format(Out, '~s', [String]), Out, Encoding, Len).
+length_of(atom(Atom, Encoding), Len) =>
+    print_length(format(Out, '~a', [Atom]), Out, Encoding, Len).
+length_of(file(File), Len) =>
     size_file(File, Len).
-length_of(memory_file(Handle), Len) :-
-    !,
+length_of(memory_file(Handle), Len) =>
     size_memory_file(Handle, Len, octet).
-length_of(html_tokens(Tokens), Len) :-
-    !,
+length_of(html_tokens(Tokens), Len) =>
     html_print_length(Tokens, Len).
-length_of(html(Tokens), Len) :-     % deprecated
-    !,
-    html_print_length(Tokens, Len).
-length_of(bytes(Bytes), Len) :-
-    !,
-    (   string(Bytes)
-    ->  string_length(Bytes, Len)
-    ;   length(Bytes, Len)          % assuming a list of 0..255
-    ).
-length_of(Len, Len).
+length_of(html(Tokens, Encoding), Len) =>
+    print_length(print_html(Out, Tokens), Out, Encoding, Len).
+length_of(bytes(Bytes), Len) =>
+    print_length(format(Out, '~s', [Bytes]), Out, octet, Len).
+length_of(Num, Len), integer(Num) =>
+    Len = Num.
 
+print_length(Goal, Out, Encoding, Len) :-
+    setup_call_cleanup(
+        open_null_stream(Out),
+        ( set_stream(Out, encoding(Encoding)),
+          call(Goal),
+          byte_count(Out, Len)
+        ),
+        close(Out)).
 
 %!  content_range(+Unit:atom, +From:int, +RangeEnd:int, +Size:int)// is det
 %
@@ -1762,7 +1785,7 @@ header_field(Name, Value) -->
 
 %!  read_field_value(-Codes)//
 %
-%   Read a field eagerly upto the next whitespace
+%   Read a field eagerly up to the next whitespace
 
 read_field_value([H|T]) -->
     [H],
@@ -1819,6 +1842,8 @@ send_request_header(Out, String) :-
 %     * content_type
 %     Parsed into media(Type/SubType, Attributes), where Attributes
 %     is a list of Name=Value pairs.
+%     * expires
+%     Parsed into a time stamp using http_timestamp/2.
 %
 %   As some fields are already parsed in the `Request`, this predicate
 %   is a no-op when called on an already parsed field.
@@ -1830,6 +1855,8 @@ http_parse_header_value(Field, Value, Prolog) :-
     known_field(Field, _, Type),
     (   already_parsed(Type, Value)
     ->  Prolog = Value
+    ;   parse_header_value_atom(Field, Value, Prolog)
+    ->  true
     ;   to_codes(Value, Codes),
         parse_header_value(Field, Codes, Prolog)
     ).
@@ -1846,6 +1873,7 @@ already_parsed(Term, V)       :- subsumes_term(Term, V).
 
 known_field(content_length,      true,  integer).
 known_field(status,              true,  integer).
+known_field(expires,             false, number).
 known_field(cookie,              true,  list(_=_)).
 known_field(set_cookie,          true,  list(set_cookie(_Name,_Value,_Options))).
 known_field(host,                true,  _Host:_Port).
@@ -1880,6 +1908,16 @@ field_to_prolog(Field, Codes, Prolog) :-
 field_to_prolog(_, Codes, Atom) :-
     atom_codes(Atom, Codes).
 
+%!  parse_header_value_atom(+Field, +ValueAtom, -Value) is semidet.
+%
+%   As parse_header_value/3, but avoid translation to codes.
+
+parse_header_value_atom(content_length, Atom, ContentLength) :-
+    atomic(Atom),
+    atom_number(Atom, ContentLength).
+parse_header_value_atom(expires, Atom, Stamp) :-
+    http_timestamp(Stamp, Atom).
+
 %!  parse_header_value(+Field, +ValueCodes, -Value) is semidet.
 %
 %   Parse the value text of an HTTP   field into a meaningful Prolog
@@ -1887,6 +1925,8 @@ field_to_prolog(_, Codes, Atom) :-
 
 parse_header_value(content_length, ValueChars, ContentLength) :-
     number_codes(ContentLength, ValueChars).
+parse_header_value(expires, ValueCodes, Stamp) :-
+    http_timestamp(Stamp, ValueCodes).
 parse_header_value(status, ValueChars, Code) :-
     (   phrase(" ", L, _),
         append(Pre, L, ValueChars)
@@ -2362,11 +2402,24 @@ rfc_date(Time, String, Tail) :-
                 '%a, %d %b %Y %T GMT',
                 Date, posix).
 
-%!  http_timestamp(+Time:timestamp, -Text:atom) is det.
+%!  http_timestamp(?Time:timestamp, ?Text:atom) is det.
 %
-%   Generate a description of a Time in HTTP format (RFC1123)
+%   Convert between a SWI-Prolog time stamp and  a string in HTTP format
+%   (RFC1123). When parsing, it  accepts   RFC1123,  RFC1036 and ASCTIME
+%   formats. See parse_time/3.
+%
+%   @error syntax_error(http_timestamp(Text)) if the string cannot
+%   be parsed.
 
-http_timestamp(Time, Atom) :-
+http_timestamp(Time, Text), nonvar(Text) =>
+    (   parse_time(Text, _Format, Time0)
+    ->  (   var(Time)
+        ->  Time = Time0
+        ;   Time =:= Time0
+        )
+    ;   syntax_error(http_timestamp(Text))
+    ).
+http_timestamp(Time, Atom), number(Time) =>
     stamp_date_time(Time, Date, 'UTC'),
     format_time(atom(Atom),
                 '%a, %d %b %Y %T GMT',
@@ -2698,7 +2751,7 @@ header(_) -->
 
 %!  address//
 %
-%   Emit the HTML for the server address on behalve of error and
+%   Emit the HTML for the server address on behalf of error and
 %   status messages (non-200 replies).  Default is
 %
 %       ==
