@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2007-2020, University of Amsterdam
+    Copyright (c)  2007-2023, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -50,7 +51,6 @@
             atom_json_dict/3            % ?Atom, ?JSONDict, +Options
           ]).
 :- use_module(library(record)).
-:- use_module(library(memfile)).
 :- use_module(library(error)).
 :- use_module(library(option)).
 :- use_module(library(lists)).
@@ -58,7 +58,8 @@
 :- use_foreign_library(foreign(json)).
 
 :- multifile
-    json_write_hook/4.                          % +Term, +Stream, +State, +Options
+    json_write_hook/4,                  % +Term, +Stream, +State, +Options
+    json_dict_pairs/2.                  % +Dict, -Pairs
 
 :- predicate_options(json_read/3, 3,
                      [ null(ground),
@@ -141,16 +142,14 @@ default_json_dict_options(
 %   _write_ mode (JSONTerm to Atom), the option
 %
 %       * as(Type)
-%       defines the output type, which is one of =atom= (default),
-%       =string=, =codes= or =chars=.
+%       defines the output type, which is one of `atom` (default),
+%       `string`, `codes` or `chars`.
 
 atom_json_term(Atom, Term, Options) :-
     ground(Atom),
     !,
     setup_call_cleanup(
-        ( atom_to_memory_file(Atom, MF),
-          open_memory_file(MF, read, In, [free_on_close(true)])
-        ),
+        open_string(Atom, In),
         json_read(In, Term, Options),
         close(In)).
 atom_json_term(Result, Term, Options) :-
@@ -259,6 +258,16 @@ json_read(Stream, Term, Options) :-
     ).
 
 json_value_top(Stream, Term, Options) :-
+    stream_property(Stream, type(binary)),
+    !,
+    setup_call_cleanup(
+        set_stream(Stream, encoding(utf8)),
+        json_value_top_(Stream, Term, Options),
+        set_stream(Stream, type(binary))).
+json_value_top(Stream, Term, Options) :-
+    json_value_top_(Stream, Term, Options).
+
+json_value_top_(Stream, Term, Options) :-
     get_code(Stream, C0),
     ws(C0, Stream, C1),
     (   C1 == -1
@@ -375,17 +384,47 @@ escape(0'n, _, 0'\n) :- !.
 escape(0'r, _, 0'\r) :- !.
 escape(0't, _, 0'\t) :- !.
 escape(0'u, Stream, C) :-
-    !,
-    get_code(Stream, C1),
-    get_code(Stream, C2),
-    get_code(Stream, C3),
-    get_code(Stream, C4),
-    code_type(C1, xdigit(D1)),
-    code_type(C2, xdigit(D2)),
-    code_type(C3, xdigit(D3)),
-    code_type(C4, xdigit(D4)),
+    get_XXXX(Stream, H),
+    (   hi_surrogate(H)
+    ->  get_surrogate_tail(Stream, H, C)
+    ;   C = H
+    ).
+
+get_XXXX(Stream, C) :-
+    get_xdigit(Stream, D1),
+    get_xdigit(Stream, D2),
+    get_xdigit(Stream, D3),
+    get_xdigit(Stream, D4),
     C is D1<<12+D2<<8+D3<<4+D4.
 
+get_xdigit(Stream, D) :-
+    get_code(Stream, C),
+    code_type(C, xdigit(D)),
+    !.
+get_xdigit(Stream, _) :-
+    syntax_error(hexdigit_expected, Stream).
+
+get_surrogate_tail(Stream, Hi, Codepoint) :-
+    (   get_code(Stream, 0'\\),
+        get_code(Stream, 0'u),
+        get_XXXX(Stream, Lo),
+        surrogate([Hi, Lo], Codepoint)
+    ->  true
+    ;   syntax_error(illegal_surrogate_pair, Stream)
+    ).
+
+
+hi_surrogate(C) :-
+    C >= 0xD800, C < 0xDC00.
+
+lo_surrogate(C) :-
+    C >= 0xDC00, C < 0xE000.
+
+surrogate([Hi, Lo], Codepoint) :-
+    hi_surrogate(Hi),
+    lo_surrogate(Lo),
+    Codepoint is (Hi - 0xD800) * 0x400 + (Lo - 0xDC00) + 0x10000.
+
 json_read_constant(0't, Stream, true) :-
     !,
     must_see(`rue`, Stream, true).
@@ -507,7 +546,7 @@ stream_error_context(Stream, stream(Stream, Line, LinePos, CharNo)) :-
 %       single-line output.
 %
 %       * step(+Step)
-%       Indentation increnment for next level.  Default is 2.
+%       Indentation increment for next level.  Default is 2.
 %
 %       * tab(+TabDistance)
 %       Distance between tab-stops.  If equal to Step, layout
@@ -518,7 +557,7 @@ stream_error_context(Stream, stream(Stream, Line, LinePos, CharNo)) :-
 %       print them as a JSON string.  The default raises a type
 %       error.  Note that this option only makes sense if you can
 %       guarantee that the passed value is not an otherwise valid
-%       Prolog reporesentation of a Prolog term.
+%       Prolog representation of a Prolog term.
 %
 %   If a string is  emitted,  the   sequence  =|</|=  is  emitted as
 %   =|<\/|=. This is valid  JSON  syntax   which  ensures  that JSON
@@ -535,21 +574,24 @@ stream_error_context(Stream, stream(Stream, Line, LinePos, CharNo)) :-
 %   point numbers are emitted.
 %
 %   Note that this hook is shared by all   users  of this library. It is
-%   generally  adviced  to  map  a  unique    compound   term  to  avoid
+%   generally  advised  to  map  a  unique    compound   term  to  avoid
 %   interference with normal output.
 %
 %   @arg State and Options are opaque handles to the current output
 %   state and settings.  Future versions may provide documented access
-%   to these terms.  Currently it is adviced to ignore these arguments.
-
+%   to these terms.  Currently it is advised to ignore these arguments.
 
+%!  json_dict_pairs(+Dict, -Pairs) is semidet.
+%
+%   This hook may be used to order the   keys of an object. If it fails,
+%   dict_pairs/3 is used which produces an ordered list of keys.
 
 :- record json_write_state(indent:nonneg = 0,
-                       step:positive_integer = 2,
-                       tab:positive_integer = 8,
-                       width:nonneg = 72,
-                       serialize_unknown:boolean = false
-                      ).
+                           step:positive_integer = 2,
+                           tab:positive_integer = 8,
+                           width:nonneg = 72,
+                           serialize_unknown:boolean = false
+                          ).
 
 json_write(Stream, Term) :-
     json_write(Stream, Term, []).
@@ -566,9 +608,9 @@ json_write_term(json(Pairs), Stream, State, Options) :-
     !,
     json_write_object(Pairs, Stream, State, Options).
 json_write_term(Dict, Stream, State, Options) :-
-    is_dict(Dict),
+    is_dict(Dict, Tag),
     !,
-    dict_pairs(Dict, Tag, Pairs0),
+    json_pairs(Dict, Pairs0),
     (   nonvar(Tag),
         json_options_tag(Options, Name),
         Name \== ''
@@ -645,6 +687,12 @@ json_write_term(AnyTerm, Stream, State, _Options) :-
     ;   type_error(json_term, AnyTerm)
     ).
 
+json_pairs(Dict, Pairs) :-
+    json_dict_pairs(Dict, Pairs),
+    !.
+json_pairs(Dict, Pairs) :-
+    dict_pairs(Dict, _, Pairs).
+
 json_write_object(Pairs, Stream, State, Options) :-
     space_if_not_at_left_margin(Stream, State),
     write(Stream, '{'),
@@ -672,7 +720,10 @@ write_pairs_hor([H|T], Stream, State, Options) :-
     json_write_term(Value, Stream, State, Options),
     (   T == []
     ->  true
-    ;   write(Stream, ', '),
+    ;   (   json_write_state_width(State, 0)
+        ->  write(Stream, ',')
+        ;   write(Stream, ', ')
+        ),
         write_pairs_hor(T, Stream, State, Options)
     ).
 
@@ -1032,7 +1083,7 @@ make_json_dict_options(Options, Record, RestOptions) :-
 %
 %   Convert  between  textual  representation  and    a   JSON  term
 %   represented as a dict. Options are as for json_read/3.
-%   In _write_ mode, the addtional option
+%   In _write_ mode, the additional option
 %
 %       * as(Type)
 %       defines the output type, which is one of =atom=,
@@ -1070,3 +1121,6 @@ json_syntax_error(illegal_comment) -->
     [ 'Illegal comment' ].
 json_syntax_error(illegal_string_escape) -->
     [ 'Illegal escape sequence in string' ].
+json_syntax_error(illegal_surrogate_pair) -->
+    [ 'Illegal escaped surrogate pair in string' ].
+
