@@ -41,11 +41,20 @@
             read_nquad/2,               % +Stream, -Quad
             read_ntuple/2               % +Stream, -TripleOrQuad
           ]).
-:- use_module(library(record)).
-:- use_module(library(uri)).
-:- use_module(library(option)).
-:- use_module(library(http/http_open)).
-:- use_module(library(semweb/rdf_db)).
+:- if(exists_source(library(semweb/rdf_db))).
+:- use_module(library(semweb/rdf_db),
+              [rdf_transaction/2,rdf_set_graph/2,rdf_assert/4]).
+:- endif.
+:- use_module(library(record),[(record)/1, op(_,_,record)]).
+
+:- autoload(library(error),[domain_error/2]).
+:- autoload(library(memfile),
+	    [atom_to_memory_file/2,open_memory_file/4]).
+:- autoload(library(option),[option/3,option/2]).
+:- autoload(library(uri),
+	    [uri_file_name/2,uri_is_global/1,uri_normalized/2]).
+:- autoload(library(http/http_open),[http_open/3]).
+
 :- use_foreign_library(foreign(ntriples)).
 
 /** <module> Process files in the RDF N-Triples format
@@ -145,7 +154,7 @@ support the format =ntriples= and =nquads=.
 %     * graph(+Graph)
 %     For rdf_read_nquads/3, this defines the graph associated
 %     to _triples_ loaded from the input.  For rdf_read_ntriples/3
-%     this opion is ignored.
+%     this option is ignored.
 %
 %   @arg Triples is a list of rdf(Subject, Predicate, Object)
 %   @arg Quads is a list of rdf(Subject, Predicate, Object, Graph)
@@ -364,6 +373,7 @@ init_state(In, Options, State) :-
                  *          RDF-DB HOOK         *
                  *******************************/
 
+:- if(current_predicate(rdf_transaction/2)).
 :- multifile
     rdf_db:rdf_load_stream/3,
     rdf_db:rdf_file_type/2.
@@ -400,7 +410,6 @@ assert_tuple(rdf(S,P,O), Graph) :-
 assert_tuple(rdf(S,P,O,Graph), _) :-
     rdf_assert(S,P,O,Graph).
 
-
 %!  rdf_db:rdf_file_type(+Extension, -Format)
 %
 %   Bind the ntriples reader to  files   with  the  extensions =nt=,
@@ -410,3 +419,4 @@ rdf_db:rdf_file_type(nt,       ntriples).
 rdf_db:rdf_file_type(ntriples, ntriples).
 rdf_db:rdf_file_type(nq,       nquads).
 rdf_db:rdf_file_type(nquads,   nquads).
+:- endif.
