@@ -389,29 +389,6 @@ getDisplayGraphical(Graphical gr)
 }
 
 
-Monitor
-getMonitorGraphical(Graphical gr)
-{ DisplayObj d = getDisplayGraphical(gr);
-  Point pt = NIL;
-  Monitor mon = NULL;
-
-  ComputeGraphical(gr);
-
-  if ( (d  = getDisplayGraphical(gr)) &&
-       (pt = getDisplayPositionGraphical(gr)) )
-  { Area a = tempObject(ClassArea,
-			pt->x, pt->y, gr->area->w, gr->area->h,
-			EAV);
-
-    mon = getMonitorDisplay(d, a);
-    considerPreserveObject(a);
-  }
-  doneObject(pt);
-
-  answer(mon);
-}
-
-
 Application
 getApplicationGraphical(Graphical gr)
 { FrameObj fr = getFrameGraphical(gr);
@@ -693,8 +670,7 @@ selection_bubble(int x, int y, int w, int h, int wx, int wy)
   int bx = x + (w - bw) * wx / 2;
   int by = y + (h - bh) * wy / 2;
 
-  r_fill(bx, by, bw, bh, BLACK_COLOUR);
-/*r_complement(bx, by, bw, bh);*/
+  r_fill(bx, by, bw, bh, NAME_foreground);
 }
 
 
@@ -901,38 +877,6 @@ RedrawAreaGraphical(Any obj, Area area)
   if ( gr->selected == ON )
     qadSendv(gr, NAME_paintSelected, 0, NULL);
 
-  succeed;
-}
-
-
-status
-flushGraphical(Any gr)
-{ PceWindow sw;
-
-  if ( (sw = getWindowGraphical(gr)) )
-    flushWindow(sw);
-
-  succeed;
-}
-
-
-status
-synchroniseGraphical(Graphical gr, BoolObj always)
-{ DisplayObj d;
-  static long last;
-
-  if ( always != ON )
-  { long now = mclock();
-
-    if ( now - last < 50 )
-      succeed;
-
-    last = now;
-  }
-
-  if ( (d = getDisplayGraphical(gr)) )
-    synchroniseDisplay(d);
-
   succeed;
 }
 
@@ -2091,8 +2035,6 @@ cursorGraphical(Graphical gr, CursorObj cursor)
   if ( w )
     updateCursorWindow(w);
 
-  flushGraphical(gr);
-
   succeed;
 }
 
@@ -3382,10 +3324,6 @@ static senddecl send_graphical[] =
      DEFAULT, "Erase from device"),
   SM(NAME_key, 1, "name", keyGraphical,
      NAME_accelerator, "Accelerator-key pressed (fail)"),
-  SM(NAME_flush, 0, NULL, flushGraphical,
-     NAME_animate, "Flush changes to the display"),
-  SM(NAME_synchronise, 1, "[always=bool]", synchroniseGraphical,
-     NAME_animate, "->flush and process all events"),
   SM(NAME_apply, 1, "[bool]", virtualObject,
      NAME_apply, "Virtual method"),
   SM(NAME_restore, 0, NULL, virtualObject,
@@ -3656,8 +3594,6 @@ static getdecl get_graphical[] =
      NAME_organisation, "Deepest device both are displayed on"),
   GM(NAME_display, 0, "display", NULL, getDisplayGraphical,
      NAME_organisation, "Display graphical is displayed on"),
-  GM(NAME_monitor, 0, "monitor", NULL, getMonitorGraphical,
-     NAME_organisation, "Monitor graphical is displayed on"),
   GM(NAME_application, 0, "application", NULL, getApplicationGraphical,
      NAME_organisation, "Application my frame belongs too"),
   GM(NAME_distance, 1, "int", "graphical", getDistanceGraphical,
