@@ -2,7 +2,7 @@
 
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        wielemak@science.uva.nl
-    WWW:           http://www.swi-prolog.org/projects/xpce/
+    WWW:           http://www.swi-prolog.org/packages/xpce/
     Copyright (c)  1985-2005, University of Amsterdam
     All rights reserved.
 
@@ -37,7 +37,7 @@
 #include <h/unix.h>
 #include <h/text.h>
 
-static void	ps_put_string(String);
+static void	ps_put_string(PceString);
 static int	postscriptImage(Image, Int, int iscolor);
 static int	header(Any, Area, BoolObj);
 static int	footer(void);
@@ -129,7 +129,7 @@ recognised:
   Option   Argument	Description
     ~~        -		Output a tilde
     ~s	    char *	Output literal string
-    ~a	    String	Output String as PostScript string
+    ~a	    PceString	Output PceString as PostScript string
     ~c	    Point	Output <-x and <-y of the point
     ~d	    Int		Output an integer
     ~D	    int		Output an integer
@@ -173,7 +173,7 @@ _output(char *fm, va_list args)
 			return;
 	  case 's':	putString(va_arg(args, char *));
 			continue;
-	  case 'd':	sprintf(tmp, INTPTR_FORMAT, valInt(va_arg(args, Int)));
+	  case 'd':	sprintf(tmp, "%" PRIdPTR, valInt(va_arg(args, Int)));
 			putString(tmp);
 			continue;
 	  case 'D':	sprintf(tmp, "%d", va_arg(args, int));
@@ -189,26 +189,26 @@ _output(char *fm, va_list args)
 			continue;
 		      }
 	  case 'm':   { Graphical gr = va_arg(args, Graphical);
-			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT " moveto",
+			sprintf(tmp, "%" PRIdPTR " " "%" PRIdPTR " moveto",
 				valInt(gr->area->x),
 				valInt(gr->area->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'c':   { Point p = va_arg(args, Point);
-			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT,
+			sprintf(tmp, "%" PRIdPTR " " "%" PRIdPTR,
 				valInt(p->x), valInt(p->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 't':   { Figure f = va_arg(args, Figure);
-			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT " translate",
+			sprintf(tmp, "%" PRIdPTR " " "%" PRIdPTR " translate",
 				valInt(f->offset->x), valInt(f->offset->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'p':   { Graphical gr = va_arg(args, Graphical);
-			sprintf(tmp, INTPTR_FORMAT, valInt(gr->pen));
+			sprintf(tmp, "%" PRIdPTR, valInt(gr->pen));
 			putString(tmp);
 			continue;
 		      }
@@ -233,7 +233,7 @@ _output(char *fm, va_list args)
 
 			continue;
 		      }
-	  case 'a':	ps_put_string(va_arg(args, String));
+	  case 'a':	ps_put_string(va_arg(args, PceString));
 			continue;
 	  case 'N':	putString(strName(va_arg(args, Name)));
 			continue;
@@ -262,25 +262,25 @@ _output(char *fm, va_list args)
 		      }
 	  case 'x':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->x));
+			sprintf(tmp, "%" PRIdPTR, valInt(gr->area->x));
 			putString(tmp);
 			continue;
 		      }
 	  case 'y':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->y));
+			sprintf(tmp, "%" PRIdPTR, valInt(gr->area->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'w':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->w));
+			sprintf(tmp, "%" PRIdPTR, valInt(gr->area->w));
 			putString(tmp);
 			continue;
 		      }
 	  case 'h':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->h));
+			sprintf(tmp, "%" PRIdPTR, valInt(gr->area->h));
 			putString(tmp);
 			continue;
 		      }
@@ -310,7 +310,7 @@ ps_output(char *fm, ...)
 
 
 static void
-ps_put_string(String s)
+ps_put_string(PceString s)
 { wint_t c;
   int i, size = s->s_size;
 
@@ -1255,7 +1255,7 @@ drawPostScriptCircle(Circle c, Name hb)
 
 
 status
-drawPostScriptEllipse(Ellipse e, Name hb)
+drawPostScriptEllipse(EllipseObj e, Name hb)
 { if ( hb == NAME_head )
   { psdef(NAME_draw);
     psdef(NAME_nodash);
@@ -1538,7 +1538,7 @@ drawPostScriptArrow(Arrow a, Name hb)
 
 
 status
-drawPostScriptArc(Arc a, Name hb)
+drawPostScriptArc(ArcObj a, Name hb)
 { if ( hb == NAME_head )
   { psdef(NAME_draw);
     psdef(NAME_arcpath);
@@ -1666,7 +1666,7 @@ drawPostScriptImage(Image image, Name hb)
 
 status
 drawPostScriptText(TextObj t, Name hb)
-{ String s = &t->string->data;
+{ PceString s = &t->string->data;
 
   if ( s[0].s_size > 0 )		/* i.e. non-empty */
   { int x, y, w;
@@ -1753,13 +1753,3 @@ postscriptFrame(FrameObj fr, Name hb)
   } else
     return ws_postscript_frame(fr, TRUE);
 }
-
-
-status
-postscriptDisplay(DisplayObj d, Name hb)
-{ if ( hb == NAME_head )
-  { psdef(NAME_rgbimage);
-    succeed;
-  } else
-    return ws_postscript_display(d, TRUE);
-}
