@@ -77,16 +77,15 @@ COMMON(Any)	RedrawBoxFigure(Figure f, Area area);
 COMMON(status)	makeClassFigure(Class class);
 
 /* /swi40/jan/src/pl/packages/xpce/src/gra/font.c */
-COMMON(status)	replaceFont(FontObj f, DisplayObj d);
 COMMON(status)	makeBuiltinFonts(void);
 COMMON(Int)	getWidthFont(FontObj f, CharArray txt);
 COMMON(Int)	getAdvanceFont(FontObj f, CharArray txt);
 COMMON(Int)	getExFont(FontObj f);
+COMMON(Int)	getAvgCharWidthFont(FontObj f);
 COMMON(Int)	getHeightFont(FontObj f);
 COMMON(Int)	getAscentFont(FontObj f);
 COMMON(Int)	getDescentFont(FontObj f);
 COMMON(BoolObj)	getFixedWidthFont(FontObj f);
-COMMON(BoolObj)	getB16Font(FontObj f);
 COMMON(status)	makeClassFont(Class class);
 
 /* /swi40/jan/src/pl/packages/xpce/src/gra/format.c */
@@ -110,7 +109,6 @@ COMMON(Graphical) getRootGraphical(Graphical gr);
 COMMON(PceWindow) getWindowGraphical(Graphical gr);
 COMMON(FrameObj) getFrameGraphical(Graphical gr);
 COMMON(DisplayObj) getDisplayGraphical(Graphical gr);
-COMMON(Monitor)	getMonitorGraphical(Graphical gr);
 COMMON(Application) getApplicationGraphical(Graphical gr);
 COMMON(Device)	getCommonDeviceGraphical(Graphical gr1, Graphical gr2);
 COMMON(int)	get_extension_margin_graphical(Graphical gr);
@@ -288,7 +286,6 @@ COMMON(status)	drawPostScriptBitmap(BitmapObj bm, Name hb);
 COMMON(status)	drawPostScriptImage(Image image, Name hb);
 COMMON(status)	drawPostScriptText(TextObj t, Name hb);
 COMMON(status)	postscriptFrame(FrameObj fr, Name hb);
-COMMON(status)	postscriptDisplay(DisplayObj d, Name hb);
 
 /* /swi40/jan/src/pl/packages/xpce/src/gra/scrollbar.c */
 COMMON(Int)	getMarginScrollBar(ScrollBar sb);
