@@ -303,10 +303,10 @@ __pce_export int	pceEnumElements(PceObject collection,
 		 *******************************/
 
 __pce_export bool pceMTinit(void);
+__pce_export void pceMTdetach(void);
 __pce_export void pceMTLock(void);
 __pce_export void pceMTUnlock(void);
 __pce_export bool pceMTTryLock(void);
-__pce_export bool pceMTdetach(void);
 __pce_export int  pceMTUnlockAll(void);
 __pce_export void pceMTRelock(int count);
 __pce_export bool setPceThread(const char *app_name);
