@@ -1344,7 +1344,6 @@ NewClass(var)
   ABSTRACT_FUNCTION
   Name		name;			/* Name of the variable */
   Type		type;			/* Type of the variable */
-  Any		value;			/* Current value of the variable */
   Any		global_value;		/* Initial or global value */
 End;
 
@@ -1470,8 +1469,6 @@ typedef struct
 typedef struct var_environment * VarEnvironment;
 typedef struct var_extension * VarExtension;
 
-GLOBAL VarEnvironment varEnvironment;
-
 struct var_environment
 { VarEnvironment parent;
   int		 size;
@@ -1485,7 +1482,6 @@ struct var_extension
   var_binding	 bindings[BINDINGBLOCKSIZE];
 };
 
-
 #define withLocalVars(code) \
   { struct var_environment _var_env; \
  \
@@ -1514,8 +1510,7 @@ struct var_extension
       const Any *_val = (av); \
       for( _i=ac; --_i >= 0; _b++, _v++, _val++) \
       { _b->variable = *_v; \
-	_b->value = _b->variable->value; \
-	_b->variable->value = *_val; \
+	_b->value = *_val; \
 	if ( isObject(*_val) ) \
 	  addCodeReference(*_val); \
       } \
@@ -1531,15 +1526,12 @@ struct var_extension
     popVarEnvironment(); \
   }
 
-#define withReceiver(r, c, code) \
-  { Any _rs = RECEIVER->value; \
-    Any _rc = RECEIVER_CLASS->value; \
-    RECEIVER->value = (r); \
-    RECEIVER_CLASS->value = (c); \
-    code; \
-    RECEIVER_CLASS->value = _rc; \
-    RECEIVER->value = _rs; \
-  }
+#define withReceiver(r, c, code)			\
+  withLocalVars(					\
+  { assignVar(RECEIVER, (r), NAME_local);		\
+    assignVar(RECEIVER_CLASS, (c), NAME_local);		\
+    code;						\
+  })
 
 
 		/********************************
@@ -1882,6 +1874,8 @@ extern char *T_report[];		/* ->report: kind, format, args... */
 		 *******************************/
 
 #include <h/interface.h>
+#include <h/thread.h>
+
 
 		/********************************
 		*        INLINE SUPPORT		*
