@@ -368,78 +368,114 @@ checkSummaryCharp(Name classname, Name name, char *s)
     sysPce("%s (%s): Bad summary string: %s", pp(classname), pp(name), s);
 }
 
+#ifndef HAVE_STPCPY
+#define stpcpy my_stpcpy	/* avoid name conflicts */
+static char *
+stpcpy(char *dst, const char *src)
+{ char *p = mempcpy(dst, src, strlen(src));
+  *p = '\0';
+  return p;
+}
+#endif
 
+static Name
+chrName(int chr)
+{ switch(chr)
+  { case ESC:	return NAME_ESC;
+    case ' ':   return NAME_SPC;
+    case '\t':  return NAME_TAB;
+    case '\b':  return NAME_BS;
+    case '\r':  return NAME_RET;
+    case '\n':  return NAME_LFD;
+    case DEL:   return NAME_DEL;
+    default:
+      fail;
+  }
+}
+
+/** Emacs character name mapping (in this order)
+
+C-  (Control)
+M-  (Meta)
+S-  (Shift)
+s-  (Super)	// SDL3 "GUI" key: Window key or Apple Command key
+H-  (Hyper)
+A-  (Alt)
+
+*/
 
 Name
 characterName(Any chr)
-{ wchar_t buf[10];
-  int ctrl;
-  int c;
+{ char buf[100];
+  char *out  = buf;
+  bool ctrl  = false;
+  bool shift = false;
+  bool gui   = false;
+  bool meta  = false;
+  Name name  = NULL;
+  int c = EOF;
 
   if ( instanceOfObject(chr, ClassEvent) )
   { EventObj ev = chr;
 
-    if ( !isInteger(ev->id) )
-      return ev->id;
-    else
+    if ( isInteger(ev->id) )
     { c = valInt(ev->id);
-      ctrl = (valInt(ev->buttons) & BUTTON_control);
+      if ( c == ' ' )
+	name = NAME_SPC;
+    } else
+    { name = ev->id;
     }
-  } else
-  { if ( !isInteger(chr) )
-      return chr;
 
-    c = valInt(chr);
-    ctrl = FALSE;
+    int btns = valInt(ev->buttons);
+    ctrl  = (btns & BUTTON_control);
+    meta  = (btns & BUTTON_meta);
+    shift = (btns & BUTTON_shift);
+    gui   = (btns & BUTTON_gui);
+  } else
+  { if ( isInteger(chr) )
+    { c = valInt(chr);
+      name = chrName(c);
+      ctrl = (c < ' ' && !name);
+      meta = ( c >= META_OFFSET );
+    } else
+      return chr;
   }
 
+  if ( name == NAME_ESC )
+    return UTF8ToName("\\e");
+
   if ( c >= META_OFFSET )
-  { wcscpy(buf, L"\\e");
     c -= META_OFFSET;
-  } else
-    buf[0] = EOS;
 
+  if ( meta )
+    out = stpcpy(out, "\\e");	/* TBD: Turn into M- and use as second */
   if ( ctrl )
-    goto ctrl;
+    out = stpcpy(out, "\\C-");
+  if ( shift && (ctrl || meta || gui || name) )
+    out = stpcpy(out, "\\S-");
+  if ( gui )
+    out = stpcpy(out, "\\s-");
 
-  switch(c)
-  { case ESC:
-      wcscat(buf, L"\\e");
-      break;
-    case ' ':
-      wcscat(buf, L"SPC");
-      break;
-    case '\t':
-      wcscat(buf, L"TAB");
-      break;
-    case '\r':
-      wcscat(buf, L"RET");
-      break;
-    case '\n':
-      wcscat(buf, L"LFD");
-      break;
-    case DEL:
-      wcscat(buf, L"DEL");
-      break;
-    default:
-    ctrl:
-      if ( c < ' ' )
-      {	size_t l;
-
-	wcscat(buf, L"\\C-");
-	buf[l=wcslen(buf)] = tolower(c + '@');
-	buf[l+1] = EOS;
-      } else
-      { size_t l;
-
-	if ( ctrl )
-	  wcscat(buf, L"\\C-");
-        buf[l=wcslen(buf)] = c;
-	buf[l+1] = EOS;
-      }
+  if ( name )
+  { const char *s = nameToUTF8(name);
+    if ( strlen(s) > 1 && islower(s[0]) )
+    { *out++ = '<';
+      out = stpcpy(out, s);
+      *out++ = '>';
+    } else
+    { out = stpcpy(out, s);
+    }
+  } else
+  { if ( c < ' ' )
+    { int c2 = tolower(c + '@');
+      *out++ = c2;
+    } else
+    { out = utf8_put_char(out, c);
+    }
   }
 
-  return WCToName(buf, wcslen(buf));
+  *out = 0;
+  return UTF8ToName(buf);
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -650,7 +686,7 @@ swritefv(int (*out)(void*, wint_t), void *closure,
 	      else
 	      { if ( (i = checkType(argv[0], TypeInt, NIL)) )
 		  a = valInt(i);
-	        else
+		else
 		{ PutString(pp(argv[0]));
 		  argc--, argv++;
 
@@ -708,7 +744,7 @@ swritefv(int (*out)(void*, wint_t), void *closure,
 	      else
 	      { if ( (f = checkType(argv[0], TypeReal, NIL)) )
 		  a = valReal(f);
-	        else
+		else
 		{ PutString(pp(argv[0]));
 		  argc--, argv++;
 
@@ -860,6 +896,7 @@ str_writefv(PceString s, CharArray format, int argc, const Any *argv)
   s->s_size = 0;		/* dubious */
   swritefv(put_str, s, &format->data, argc, argv);
   assert(s->s_size == len);
+  (void)len;
 
   succeed;
 }
@@ -961,7 +998,7 @@ scanstr(char *str, char *fmt, Any *r)
 	    if ( !supress )
 	    { types[argn] = length|T_FLOAT;
 	      ptrs[argn]  = (void *)alloca(length == L_LONG ? sizeof(double)
-				                            : sizeof(float));
+							    : sizeof(float));
 	      argn++;
 	    }
 	    s++;
@@ -1009,91 +1046,91 @@ scanstr(char *str, char *fmt, Any *r)
     case 3:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2]);
 				      break;
     case 4:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3]); break;
+				      ptrs[3]); break;
     case 5:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4]); break;
+				      ptrs[3], ptrs[4]); break;
     case 6:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5]);
+				      ptrs[3], ptrs[4], ptrs[5]);
 				      break;
     case 7:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6]); break;
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6]); break;
     case 8:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7]); break;
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7]); break;
     case 9:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8]);
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8]);
 				      break;
     case 10:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9]);
 				      break;
     case 11:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10]);
 				      break;
     case 12:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11]);
 				      break;
     case 13:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12]);
 				      break;
     case 14:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13]);
 				      break;
     case 15:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14]);
 				      break;
     case 16:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14],
 				      ptrs[15]);
 				      break;
     case 17:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14],
 				      ptrs[15], ptrs[16]);
 				      break;
     case 18:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14],
 				      ptrs[15], ptrs[16], ptrs[17]);
 				      break;
     case 19:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14],
 				      ptrs[15], ptrs[16], ptrs[17],
 				      ptrs[18]);
 				      break;
     case 20:	ar = sscanf(str, fmt, ptrs[0], ptrs[1], ptrs[2],
-			              ptrs[3], ptrs[4], ptrs[5],
-			              ptrs[6], ptrs[7], ptrs[8],
+				      ptrs[3], ptrs[4], ptrs[5],
+				      ptrs[6], ptrs[7], ptrs[8],
 				      ptrs[9], ptrs[10], ptrs[11],
 				      ptrs[12], ptrs[13], ptrs[14],
 				      ptrs[15], ptrs[16], ptrs[17],
-			              ptrs[18], ptrs[19]);
+				      ptrs[18], ptrs[19]);
 				      break;
     default:	errorPce(NIL, NAME_tooManyArguments);
 		fail;
@@ -1405,4 +1442,3 @@ run_pce_exit_hooks(int rval)
 
   return 0;
 }
-
