@@ -88,6 +88,7 @@ COMMON(status)	makeClassClass(Class class);
 COMMON(status)	toString(Any obj, PceString s);
 COMMON(char *)	toCharp(Any obj);
 COMMON(Int)	toInteger(Any obj);
+COMMON(Num)	convertToNum(Any obj);
 COMMON(Real)	toReal(Any obj);
 COMMON(BoolObj)	toBool(Any obj);
 COMMON(Name)	toName(Any obj);
@@ -181,7 +182,7 @@ COMMON(status)	createMethod(Method m, Name name, Vector types, StringObj doc, Fu
 COMMON(status)	initialiseMethod(Method m, Name name, Vector types, Code msg, StringObj doc, SourceLocation loc, Name group);
 COMMON(Type)	getArgumentTypeMethod(Method m, Int n);
 COMMON(Method)	getInheritedFromMethod(Method m);
-COMMON(Method)	getMethodFromFunction(Any f);
+COMMON(Method)	getMethodFromFunction(Any (*f)());
 COMMON(status)	makeClassMethod(Class class);
 
 /* /home/jan/src/pl/packages/xpce/src/ker/name.c */
@@ -254,6 +255,8 @@ COMMON(Chain)	getAllHypersObject(Any obj, BoolObj create);
 COMMON(Chain)	getAllAttributesObject(Any obj, BoolObj create);
 COMMON(Chain)	getAllSendMethodsObject(Any obj, BoolObj create);
 COMMON(Chain)	getAllGetMethodsObject(Any obj, BoolObj create);
+COMMON(status)  sendMethodObject(Any obj, SendMethod m);
+COMMON(status)  getMethodObject(Any obj, GetMethod m);
 COMMON(Any)	getCloneObject(Any obj);
 COMMON(status)	clonePceSlots(Any org, Any Clone);
 COMMON(Any)	getClone2Object(Any obj);
@@ -318,7 +321,8 @@ COMMON(status)	catchErrorSignalsPce(Pce pce, BoolObj val);
 COMMON(status)	resetPce(Pce pce);
 COMMON(Any)	getObjectFromReferencePce(Pce pce, Any ref);
 COMMON(status)	makeClassPce(Class class);
-COMMON(export status) pceInitialise(int handles, const char *home, int argc, char **argv);
+COMMON(export status) pceInitialise(int handles, const char *home, const char *appdata,
+				    int argc, char **argv);
 
 /* /home/jan/src/pl/packages/xpce/src/ker/sendmethod.c */
 COMMON(SendMethod) createSendMethod(Name name, Vector types, StringObj doc, SendFunc action);
