@@ -191,6 +191,25 @@ keycode_to_name(SDL_Event *event)
     case SDLK_F10:	 return NAME_f10;
     case SDLK_F11:	 return NAME_f11;
     case SDLK_F12:	 return NAME_f12;
+
+      /* The keypad sends SDLK_KP_* events, with or without
+	 numlock.   Here we assign the functions.  Next, we
+	 use the text input distinction to see whether the
+	 keypad sends a number or a command.
+
+	 But, what is the correct mapping from keypad numbers
+	 to functions?   This works on a default US keyboard.
+       */
+    case SDLK_KP_8:	 return NAME_cursorUp;
+    case SDLK_KP_4:	 return NAME_cursorLeft;
+    case SDLK_KP_6:	 return NAME_cursorRight;
+    case SDLK_KP_2:	 return NAME_cursorDown;
+    case SDLK_KP_7:	 return NAME_cursorHome;
+    case SDLK_KP_1:	 return NAME_end;
+    case SDLK_KP_9:	 return NAME_pageUp;
+    case SDLK_KP_3:	 return NAME_pageDown;
+    case SDLK_KP_PERIOD: return NAME_DEL;
+    case SDLK_KP_ENTER:	 return NAME_RET;
   }
 
   fail;
@@ -363,6 +382,10 @@ CtoEvent(SDL_Event *event)
       break;
     }
       /* https://wiki.libsdl.org/SDL3/SDL_KeyboardEvent */
+    case SDL_EVENT_KEYMAP_CHANGED:
+      DEBUG(NAME_keyboard,
+	    Cprintf("Got SDL_EVENT_KEYMAP_CHANGED\n"));
+      fail;
     case SDL_EVENT_KEY_UP:
       lastmod = event->key.mod;
       fail;		      /* only update modifiers */
@@ -396,11 +419,16 @@ CtoEvent(SDL_Event *event)
     case SDL_EVENT_KEY_DOWN:
     { SDL_Keymod isdown = (SDL_KMOD_LCTRL|SDL_KMOD_RCTRL|SDL_KMOD_GUI);
 #ifndef __APPLE__
-      isdown |= SDL_KMOD_LALT|SDL_KMOD_RALT;
+      isdown |= SDL_KMOD_LALT;
 #endif
 
       name = keycode_to_name(event);
-      if ( !name ) fail;
+      if ( !name )
+      { DEBUG(NAME_scancode,
+	      Cprintf("Ignoring keydown.  Mod=0x%x, scancode=%d, key=0x%x\n",
+		      event->key.mod, event->key.scancode, event->key.key));
+	fail;
+      }
 
       if ( event->key.mod & isdown )
 	goto immediate_down_event;
