@@ -45,6 +45,9 @@
 #define MainWindow(fr)	     ( isNil(fr->members->head) ? (Any) fr : \
 			       fr->members->head->value )
 
+static bool	ws_draw_frame(FrameObj fr);
+
+
 WsFrame
 sdl_frame(FrameObj fr, bool create)
 { WsFrame f;
@@ -169,6 +172,7 @@ ws_create_frame(FrameObj fr)
     SDL_SetPointerProperty(props, SDL_PROP_WINDOW_CREATE_PARENT_POINTER,
 			   parent);
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_MENU_BOOLEAN, true);
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_HIDDEN_BOOLEAN, true);
 #if defined(__APPLE__) && defined(SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN)
     /* SDL on MacOS does not handle popup placement correctly on secondary displays */
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN, false);
@@ -203,6 +207,7 @@ ws_create_frame(FrameObj fr)
     SDL_PropertiesID props = SDL_CreateProperties();
     SDL_SetStringProperty(props, SDL_PROP_WINDOW_CREATE_TITLE_STRING,
 			  nameToUTF8(fr->label));
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_HIDDEN_BOOLEAN, true);
     SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_WIDTH_NUMBER, w);
     SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_HEIGHT_NUMBER, h);
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_RESIZABLE_BOOLEAN,
@@ -215,27 +220,26 @@ ws_create_frame(FrameObj fr)
     { SDL_SetPointerProperty(props, SDL_PROP_WINDOW_CREATE_PARENT_POINTER,
 			     parent);
     }
+
+    Area da = fr->display->area;	/* work_area is unreliable */
+
     if ( isOn(fr->placed) )
     {
 #ifdef __WINDOWS__
       x += GetSystemMetrics(SM_CXBORDER);
       y += GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYCAPTION);
 #endif
-      DisplayObj dsp = fr->display;
-      x += valInt(dsp->work_area->x);
-      y += valInt(dsp->work_area->y);
+      x += valInt(da->x);
+      y += valInt(da->y);
       DEBUG(NAME_frame,
 	    Cprintf("%s: passing position %d,%d\n", pp(fr), x, y));
       SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
       SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
     } else
-    { DisplayObj dsp = fr->display;
-      if ( !isOn(dsp->primary) )
-      { x = valInt(dsp->work_area->x) + (valInt(dsp->work_area->w)-w)/2;
-	y = valInt(dsp->work_area->y) + (valInt(dsp->work_area->h)-h)/2;
-	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
-	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
-      }
+    { x = valInt(da->x) + (valInt(da->w)-w)/2;
+      y = valInt(da->y) + (valInt(da->h)-h)/2;
+      SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_X_NUMBER, x);
+      SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, y);
     }
     win = SDL_CreateWindowWithProperties(props);
     SDL_DestroyProperties(props);
@@ -261,6 +265,8 @@ ws_create_frame(FrameObj fr)
     DEBUG(NAME_sdl,
 	  Cprintf("Registered window %p with id %d\n", win, f->ws_id));
 
+    ws_draw_frame(fr);
+    SDL_ShowWindow(win);
     SDL_RaiseWindow(win);
 
     succeed;
@@ -605,6 +611,10 @@ sdl_frame_event(SDL_Event *ev)
       case SDL_EVENT_WINDOW_MOVED:
       { int new_x = ev->window.data1;
 	int new_y = ev->window.data2;
+	Area da = fr->display->area;
+
+	new_x -= valInt(da->x);
+	new_y -= valInt(da->y);
 
 	assign(fr->area, x, toInt(new_x));
 	assign(fr->area, y, toInt(new_y));
@@ -813,7 +823,7 @@ ws_frame_bb(FrameObj fr, int *x, int *y, int *w, int *h)
 
 void
 ws_x_geometry_frame(FrameObj fr, Name spec, DisplayObj dsp)
-{ char *e, *s = strName(spec);
+{ char *s = strName(spec);
   int x, y, w, h, w0, h0;
   int eh;
   int dw, dh;
@@ -823,25 +833,32 @@ ws_x_geometry_frame(FrameObj fr, Name spec, DisplayObj dsp)
   Int X,Y,W,H;
   int offX=0;			/* window manager frame offset */
 
-  if ( isDefault(dsp) && (e=strchr(s, '@')) )
-  { int n = atoi(e+1);
+  if ( isDefault(dsp) )
+  { char *e = strchr(s, '@');
+    int n;
 
-    if ( !(dsp=getNth0Chain(fr->display->display_manager->members, toInt(n))) )
-      dsp = DEFAULT;
+    if ( e )
+      n = atoi(e+1);
+    else
+      n = 1;
+
+    if ( !(dsp=getMemberDisplayManager(TheDisplayManager(), toInt(n))) )
+      dsp = fr->display;
   }
 
-  if ( instanceOfObject(dsp, ClassDisplay) )
-  { Area a = (notNil(dsp->work_area) ? dsp->work_area : dsp->area);
-
-    dw = valInt(a->w);
-    dh = valInt(a->h);
-  } else
-  { dw = valInt(getWidthDisplay(fr->display));
-    dh = valInt(getHeightDisplay(fr->display));
-  }
+  Area a = dsp->area;		/* work-area seems unreliable */
+  dw = valInt(a->w);
+  dh = valInt(a->h);
 
   if ( !ws_frame_bb(fr, &x, &y, &w0, &h0) )
     return;
+
+  x -= valInt(a->x);			/* relative to display origin */
+  y -= valInt(a->y);
+  DEBUG(NAME_geometry,
+	Cprintf("%s at %d,%d,%d,%d on %s\n",
+		pp(fr), x, y, w0, h0, pp(dsp)));
+
   w = w0;
   h = h0;
   eh = h - valInt(fr->area->h);		/* height of decorations */
@@ -904,6 +921,7 @@ ws_x_geometry_frame(FrameObj fr, Name spec, DisplayObj dsp)
   if ( !(flags & WIN_NOMOVE) )
   { X = toInt(x);
     Y = toInt(y);
+    assign(fr, placed, ON);
   }
   if ( !(flags & WIN_NOSIZE) )
   { W = toInt(w);
@@ -928,7 +946,7 @@ ws_geometry_frame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
 { WsFrame wsf = fr->ws_ref;
 
   if ( wsf )
-  {  if ( notDefault(w) || notDefault(h) )
+  { if ( notDefault(w) || notDefault(h) )
     { int iw = isDefault(w) ? valInt(fr->area->w) : valInt(w);
       int ih = isDefault(h) ? valInt(fr->area->h) : valInt(h);
 
@@ -949,6 +967,11 @@ ws_geometry_frame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
     { int ix = isDefault(x) ? valInt(fr->area->x) : valInt(x);
       int iy = isDefault(y) ? valInt(fr->area->y) : valInt(y);
 
+      if ( notDefault(dsp) )
+      { ix += valInt(dsp->area->x);
+	iy += valInt(dsp->area->y);
+      }
+
 #if O_HDPX
       float scale = ws_pixel_density_display(fr);
       ix = ix/scale; iy = iy/scale;
