@@ -620,9 +620,9 @@ get_position_from_center_frame(FrameObj fr, DisplayObj d, Point pos,
   { d = CurrentDisplay(fr);
 
     if ( d )
-    { Area a = d->work_area;
-      *x = valInt(a->x) + valInt(a->w)/2;
-      *y = valInt(a->y) + valInt(a->h)/2;
+    { Area a = d->area;
+      *x = valInt(a->w)/2;
+      *y = valInt(a->h)/2;
     } else
     { *x = *y = 0;
     }
@@ -645,7 +645,7 @@ ensure_on_display(FrameObj fr, DisplayObj dsp, int *x, int *y)
   if ( isDefault(dsp) )
     dsp = CurrentDisplay(fr);
 
-  Area a = dsp->work_area;
+  Area a = dsp->area;
   rm = valInt(a->x) + valInt(a->w);
   bm = valInt(a->y) + valInt(a->h);
 
@@ -692,27 +692,28 @@ getGeometryFrame(FrameObj fr)
 
   if ( ws_frame_bb(fr, &x, &y, &ww, &wh) )	/* outer area */
   { int mx, my, mw, mh;
-    int xn=FALSE, yn=FALSE;
+    bool xn=false, yn=false;
     char buf[100];
     DisplayObj d = fr->display;
     int cw, ch;
 
+    Area a = d->area;		/* work area seems unreliable */
     cw = valInt(fr->area->w);			/* Client area */
     ch = valInt(fr->area->h);
 
     mx = my = 0;
-    mw = valInt(d->area->w);
-    mh = valInt(d->area->h);
+    mw = valInt(a->w);
+    mh = valInt(a->h);
 
     if ( x-mx > ((mx+mw) - (x+ww))*2 )	/* over 2/3th */
     { x = (mx+mw) - (x+ww);
-      xn = TRUE;
+      xn = true;
     } else
     { x -= mx;
     }
     if ( y-my > ((my+mh) - (y+wh))*2 )
     { y = (my+mh) - (y+wh);
-      yn = TRUE;
+      yn = true;
     } else
     { y -= my;
     }
@@ -725,10 +726,8 @@ getGeometryFrame(FrameObj fr)
     sprintf(buf+strlen(buf),
 	    "%s%d%s%d", xn ? "-" : "+", x, yn ? "-" : "+", y);
 
-    Int n = getIndexChain(d->display_manager->members, d);
-
-    if ( n != ONE )
-      sprintf(buf+strlen(buf), "@%" PRIdPTR, valInt(n)-1);
+    if ( d->number != ONE )
+      sprintf(buf+strlen(buf), "@%d", (int)valInt(d->number));
 
     answer(CtoName(buf));
   }
@@ -750,7 +749,6 @@ getImageFrame(FrameObj fr)
 static status
 geometryFrame(FrameObj fr, Name spec, DisplayObj dsp)
 { assign(fr, geometry, spec);
-  assign(fr, placed, ON);
 
   ws_x_geometry_frame(fr, spec, dsp);
 
@@ -762,18 +760,14 @@ static status
 setFrame(FrameObj fr, Int x, Int y, Int w, Int h, DisplayObj dsp)
 { Area a = fr->area;
 
-  if ( notDefault(dsp) )
-  { if ( notDefault(x) )
-      x = add(x, dsp->work_area->x);
-    if ( notDefault(y) )
-      y = add(y, dsp->work_area->y);
-  }
-
-  if ( createdFrame(fr) )
+  if ( ws_created_frame(fr) )
   { setArea(a, x, y, DEFAULT, DEFAULT);
     sdl_send(fr, NAME_SdlSet, false, x, y, w, h, dsp, EAV);
   } else
-  { if ( notDefault(x) || notDefault(y) )
+  { if ( notDefault(dsp) && dsp != fr->display )
+      assign(fr, display, dsp);
+
+    if ( notDefault(x) || notDefault(y) )
       assign(fr, placed, ON);
     setArea(a, x, y, w, h);
     if ( valInt(a->w) <= 0 )		/* Window systems don't like that */
@@ -1198,7 +1192,7 @@ getKeyboardFocusFrame(FrameObj fr)
       sw2 = dw->window;
     }
 
-    if ( notNil(sw2->focus) )
+    if ( notNil(sw2->focus) || notNil(sw2->keyboard_focus) )
       answer(sw2);
   }
 
@@ -1226,7 +1220,11 @@ inputWindowFrame(FrameObj fr, PceWindow iw)
 
 static status
 inputFocusFrame(FrameObj fr, BoolObj val)
-{ if ( fr->input_focus != val )
+{ DEBUG(NAME_keyboard,
+	Cprintf("inputFocusFrame(%s, %s->%s)\n",
+		pp(fr), pp(fr->input_focus), pp(val)));
+
+  if ( fr->input_focus != val )
   { PceWindow iw;
 
     assign(fr, input_focus, val);
@@ -1601,8 +1599,9 @@ reportFrame(FrameObj fr, Name kind, CharArray fmt, int argc, Any *argv)
     return sendv(reporter, NAME_report, argc+2, av);
 
   for_chain(fr->members, window,
-	    if ( !(notNil(REPORTEE->value) &&
-		   memberChain(REPORTEE->value, window)) &&
+	    Chain rv;
+	    if ( !(notNil((rv=getValueVar(REPORTEE))) &&
+		   memberChain(rv, window)) &&
 		 sendv(window, NAME_report, argc+2, av) )
 	      succeed);
 
