@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2024, University of Amsterdam
+    Copyright (c)  1997-2025, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -389,6 +389,8 @@ struct PL_global_data
     int		cgc_space_factor;	/* Max total/margin garbage */
     double	cgc_stack_factor;	/* Price to scan stack space */
     double	cgc_clause_factor;	/* Pce to scan clauses */
+    Clause	top_clause;		/* See PL_open_query() */
+    struct clause_ref top_cref;		/* Its reference */
   } clauses;
 
   struct
@@ -541,8 +543,8 @@ struct PL_local_data
     term_t	tmp;			/* tmp for errors */
     term_t	pending;		/* used by the debugger */
     term_t	fr_rewritten;		/* processed by exception_hook() */
-    int		in_hook;		/* inside exception_hook() */
-    int		processing;		/* processing an exception */
+    bool	in_hook;		/* inside exception_hook() */
+    bool	processing;		/* processing an exception */
     exception_frame *throw_environment;	/* PL_throw() environments */
   } exception;
 
@@ -577,6 +579,14 @@ struct PL_local_data
     uint64_t	frequency;		/* How often do we fire */
   } yield;
 
+#ifdef __EMSCRIPTEN__
+  struct
+  { term_t yield_request;		/* See '$await'/2 in wasm/pl-wasm.c */
+    term_t yield_result;
+    bool   yield_unified;
+  } wasm;
+#endif
+
   struct
   { uint64_t	inferences;		/* inferences in this thread */
     uint64_t	last_cputime;		/* milliseconds last CPU time */
@@ -709,10 +719,25 @@ struct PL_local_data
     int		  write_attributes;	/* how to write attvars? */
     occurs_check_t occurs_check;	/* Unify and occurs check */
     access_level_t access_level;	/* Current access level */
+    unsigned int   unknown_option;	/* OPT_UNKNOWN_* */
   } prolog_flag;
 
   struct
   { FindData	find;			/* /<ports> <goal> in tracer */
+    struct
+    { char	resume_action;		/* Restart after yield */
+      short	port;			/* Port on which we stopped */
+      bool	nodebug;		/* continue in nodebug mode */
+      struct
+      { bool	is_jump;		/* Distinguish the two REDO ports */
+	struct clause_choice chp;	/* Clause redo context */
+      } redo;
+      struct
+      { term_t	catchfr_ref;		/* Catching frame reference */
+	Stack	outofstack;		/* We are processing an out-of-stack */
+	bool	start_tracer;		/* Start debugger asap */
+      } exception;
+    } yield;
   } trace;
 
   struct findall_state *bags;		/* findall/3 store  */
@@ -773,12 +798,12 @@ struct PL_local_data
     DefinitionChain local_definitions;	/* P_THREAD_LOCAL predicates */
     int magic;				/* PL_THREAD_MAGIC (checking) */
     int exit_requested;			/* Thread is asked to exit */
+    struct _PL_thread_info_t *creator;	/* Thread that created me */
+    uint64_t creator_seq_id;		/* Seq id of creater */
 #ifdef O_PLMT
     simpleMutex scan_lock;		/* Hold for asynchronous scans */
     thread_wait_for *waiting_for;	/* thread_wait/2 info */
     alert_channel alert;		/* How to alert the thread */
-    struct _PL_thread_info_t *creator;	/* Thread that created me */
-    uint64_t creator_seq_id;		/* Seq id of creater */
     double child_cputime;		/* Time of completed children */
     uint64_t child_inferences;		/* Inferences in children */
 #endif
