@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2024, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -46,7 +47,7 @@
 
 static
 PRED_IMPL("is_list", 1, is_list, 0)
-{ if ( lengthList(A1, FALSE) >= 0 )
+{ if ( lengthList(A1, false) >= 0 )
     succeed;
 
   fail;
@@ -71,7 +72,7 @@ PRED_IMPL("$length", 2, dlength, 0)
       if ( !hasGlobalSpace(len*3) )
       { int rc;
 
-	if ( (rc=ensureGlobalSpace(len*3, ALLOW_GC)) != TRUE )
+	if ( (rc=ensureGlobalSpace(len*3, ALLOW_GC)) != true )
 	  return raiseStackOverflow(rc);
       }
 
@@ -90,7 +91,7 @@ PRED_IMPL("$length", 2, dlength, 0)
     } else if ( len == 0 )
     { return PL_unify_nil(A1);
     } else
-    { return FALSE;
+    { return false;
     }
   } else if ( PL_is_integer(A2) )
   { number i;
@@ -99,7 +100,7 @@ PRED_IMPL("$length", 2, dlength, 0)
     deRef(p);
     get_integer(*p, &i);
     if ( ar_sign_i(&i) < 0 )
-      return FALSE;
+      return false;
 
     return outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_RAISE);
   }
@@ -109,7 +110,7 @@ PRED_IMPL("$length", 2, dlength, 0)
 
 
 static
-PRED_IMPL("memberchk", 2, memberchk, 0)
+PRED_IMPL("$memberchk", 3, memberchk, 0)
 { GET_LD
   term_t ex = PL_new_term_ref();
   term_t h = PL_new_term_ref();
@@ -118,26 +119,31 @@ PRED_IMPL("memberchk", 2, memberchk, 0)
   fid_t fid;
 
   if ( !(fid=PL_open_foreign_frame()) )
-    return FALSE;
+    return false;
 
   for(;;)
   { if ( ++done % 10000 == 0 )
     { if ( PL_handle_signals() < 0 )
-	return FALSE;
+	return false;
       if ( done > usedStack(global)/(sizeof(word)*2) )
 	return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_list, A2);
     }
 
+    if ( PL_is_variable(l) )
+    { PL_close_foreign_frame(fid);
+      return PL_unify(A3, l);
+    }
+
     if ( !PL_unify_list(l, h, l) )
     { PL_close_foreign_frame(fid);
       PL_unify_nil_ex(l);
-      return FALSE;
+      return false;
     }
 
     if ( PL_unify(A1, h) )
-    { if ( foreignWakeup(ex PASS_LD) )
+    { if ( foreignWakeup(ex) )
       { PL_close_foreign_frame(fid);
-	succeed;
+	return PL_unify_nil(A3);
       } else
       { if ( !isVar(*valTermRef(ex)) )
 	  return PL_raise_exception(ex);
@@ -210,27 +216,38 @@ typedef enum
     FREE	Frees a List_Record including its ITEM.
 */
 
+typedef union
+{ Word as_ptr;
+  word as_word;
+} m64_kv;
+
 typedef struct
-{ Word term;
-  Word key;
+{ m64_kv term;
+  m64_kv key;
 } ITEM;
 
 					/* TBD: handle CMP_ERROR */
 #ifndef COMPARE_KEY
-#define COMPARE_KEY(x,y) compareStandard((x)->key, (y)->key, FALSE PASS_LD)
+#define COMPARE_KEY(x,y) compareStandard((x)->key.as_ptr, (y)->key.as_ptr, false)
 #endif
 #ifndef FREE
+/* FREE() leaves the struct as three variables on the global stack */
 #define FREE(x) \
-	{ x->next = NULL; \
-	  x->item.term = NULL; \
-	  x->item.key = NULL; \
-	}
+  { setVar(x->next.as_word);	    \
+    setVar(x->item.term.as_word);   \
+    setVar(x->item.key.as_word);    \
+  }
 #endif
 
 typedef struct List_Record *list;
-struct List_Record {
-    list next;
-    ITEM item;
+typedef union
+{ list  as_ptr;
+  word  as_word;
+} m64_list;
+
+struct List_Record
+{ m64_list next;
+  ITEM     item;
 };
 
 #define NIL (list)0
@@ -254,16 +271,16 @@ nat_sort(list data, int remove_dups, sort_order order)
   while ((p = data) != NIL)
   { /* pick up a run from the front of data, setting */
     /* p = (pointer to beginning of run), data = (rest of data) */
-    if ((q = p->next) != NIL)
+    if ((q = p->next.as_ptr) != NIL)
     { compare(c, p, q);
 
-      data = q->next;
+      data = q->next.as_ptr;
       if (c > 0)
       { r = q, q = p, p = r;
-	p->next = q;
+	p->next.as_ptr = q;
       } else if (c == remove_dups)
       {	/* c < 0 or = 0, so c = 1 impossible */
-	p->next = q->next;
+	p->next.as_ptr = q->next.as_ptr;
 	FREE(q);
 	q = p;
       }
@@ -274,15 +291,15 @@ nat_sort(list data, int remove_dups, sort_order order)
 	if (c > 0)
 	  break;
 	if (c == remove_dups)
-	{ s = r->next;
+	{ s = r->next.as_ptr;
 	  FREE(r);
 	  r = s;
 	} else
-	{ q->next = r, q = r, r = r->next;
+	{ q->next.as_ptr = r, q = r, r = r->next.as_ptr;
 	}
       }
 
-      q->next = NIL;
+      q->next.as_ptr = NIL;
       data = r;
     } else
     { data = NIL;
@@ -298,18 +315,18 @@ nat_sort(list data, int remove_dups, sort_order order)
 	compare(c, q, p);
 
 	if (c <= 0)
-	{ r->next = q, r = q, q = q->next;
+	{ r->next.as_ptr = q, r = q, q = q->next.as_ptr;
 	  if (c == remove_dups)
-	  { s = p->next;
+	  { s = p->next.as_ptr;
 	    FREE(p);
 	    p = s;
 	  }
 	} else
-	{ r->next = p, r = p, p = p->next;
+	{ r->next.as_ptr = p, r = p, p = p->next.as_ptr;
 	}
       }
-      r->next = q ? q : p;
-      p = header.next;
+      r->next.as_ptr = q ? q : p;
+      p = header.next.as_ptr;
     }
 
 	 /* push the merged run onto the stack */
@@ -329,26 +346,27 @@ nat_sort(list data, int remove_dups, sort_order order)
       compare(c, q, p);
 
       if (c <= 0)
-      { r->next = q, r = q, q = q->next;
+      { r->next.as_ptr = q, r = q, q = q->next.as_ptr;
 	if (c == remove_dups)
-	{ s = p->next;
+	{ s = p->next.as_ptr;
 	  FREE(p);
 	  p = s;
 	}
       } else
-      { r->next = p, r = p, p = p->next;
+      { r->next.as_ptr = p, r = p, p = p->next.as_ptr;
       }
     }
-    r->next = q ? q : p;
-    p = header.next;
+    r->next.as_ptr = q ? q : p;
+    p = header.next.as_ptr;
   }
 
   return p;
 }
 
 
+#define extract_key(p1, argc, argv, pair) LDFUNC(extract_key, p1, argc, argv, pair)
 static Word
-extract_key(Word p1, int argc, const word *argv, int pair ARG_LD)
+extract_key(DECL_LD Word p1, int argc, const word *argv, int pair)
 { if ( pair )
   { if ( hasFunctor(*p1, FUNCTOR_minus2) )
     { p1 = argTermP(*p1, 0);
@@ -370,15 +388,15 @@ extract_key(Word p1, int argc, const word *argv, int pair ARG_LD)
       { if ( termIsDict(*p1) )
 	{ Word vp;
 
-	  if ( (vp = dict_lookup_ptr(*p1, argv[0] PASS_LD)) )
+	  if ( (vp = dict_lookup_ptr(*p1, argv[0], NULL)) )
 	  { p1 = vp;
 	    goto next;
 	  }
 	  existence = ATOM_key;
 	  goto err_exists;
 	} else if ( isInteger(argv[0]) )
-	{ int arity = arityTerm(*p1);
-	  int an = valInt(argv[0]);
+	{ size_t arity = arityTerm(*p1);
+	  sword an = valInt(argv[0]);
 
 	  if ( an <= arity )
 	  { p1 = argTermP(*p1, an-1);
@@ -422,18 +440,21 @@ typedef enum
   SORT_NOSORT
 } list_sort;
 
+#define prolog_list_to_sort_list(t, remove_dups, argc, argv, pair, lp, end) \
+	LDFUNC(prolog_list_to_sort_list, t, remove_dups, argc, argv, pair, lp, end)
+
 static list_sort
-prolog_list_to_sort_list(term_t t,		/* input list */
+prolog_list_to_sort_list(DECL_LD term_t t,		/* input list */
 			 int remove_dups,	/* allow to be cyclic */
 			 int argc, const word *argv, int pair, /* find key */
-			 list *lp, Word *end ARG_LD)	/* result list */
+			 list *lp, Word *end)	/* result list */
 { Word l, tail;
   list p;
   intptr_t len;
   int rc;
 
   l = valTermRef(t);
-  len = skip_list(l, &tail PASS_LD);
+  len = skip_list(l, &tail);
   if ( !(isNil(*tail) ||			/* proper list */
 	 (isList(*tail) && remove_dups)) )	/* sort/2 on cyclic list */
   { if ( isVar(*tail) )
@@ -450,7 +471,7 @@ prolog_list_to_sort_list(term_t t,		/* input list */
     return SORT_NOSORT;
 
   if ( !hasGlobalSpace(len*3) )
-  { if ( (rc=ensureGlobalSpace(len*3, ALLOW_GC)) != TRUE )
+  { if ( (rc=ensureGlobalSpace(len*3, ALLOW_GC)) != true )
     { raiseStackOverflow(rc);
       return SORT_ERR;
     }
@@ -462,23 +483,23 @@ prolog_list_to_sort_list(term_t t,		/* input list */
 
   deRef(l);
   while(len-- > 0)
-  { p->item.term = HeadList(l);
-    deRef(p->item.term);
-    p->item.key = extract_key(p->item.term, argc, argv, pair PASS_LD);
+  { p->item.term.as_ptr = HeadList(l);
+    deRef(p->item.term.as_ptr);
+    p->item.key.as_ptr = extract_key(p->item.term.as_ptr, argc, argv, pair);
 
-    if ( unlikely(!p->item.key) )
+    if ( unlikely(!p->item.key.as_ptr) )
       return SORT_ERR;
 
     l = TailList(l);
     deRef(l);
     if ( len > 0 )
     { assert(isList(*l));
-      p->next = p+1;
+      p->next.as_ptr = p+1;
       p++;
     }
   }
 
-  p->next = NULL;
+  p->next.as_ptr = NULL;
   *end = (Word)(p+1);
 
   return SORT_SORT;
@@ -489,16 +510,17 @@ static void
 put_sort_list(term_t l, list sl)
 { GET_LD
 
+  IS_WORD_ALIGNED(sl);
   *valTermRef(l) = consPtr(sl, TAG_COMPOUND|STG_GLOBAL);
 
   for(;;)
-  { list n = sl->next;
+  { list n = sl->next.as_ptr;
     Word p = (Word)sl;
 
-    n = sl->next;
+    n = sl->next.as_ptr;
 					/* see also linkVal() */
-    p[1] = (needsRef(*sl->item.term) ? makeRefG(sl->item.term)
-				     : *sl->item.term);
+    p[1] = (needsRef(*sl->item.term.as_ptr) ? makeRefG(sl->item.term.as_ptr)
+					    : *sl->item.term.as_ptr);
     p[0] = FUNCTOR_dot2;
     if ( n )
     { p[2] = consPtr(n, TAG_COMPOUND|STG_GLOBAL);
@@ -511,25 +533,30 @@ put_sort_list(term_t l, list sl)
 }
 
 
+#define pl_nat_sort(in, out, remove_dups, order, argc, argv, pair) \
+	LDFUNC(pl_nat_sort, in, out, remove_dups, order, argc, argv, pair)
+
 static int
-pl_nat_sort(term_t in, term_t out,
+pl_nat_sort(DECL_LD term_t in, term_t out,
 	    int remove_dups, sort_order order,
-	    int argc, const word *argv, int pair
-	    ARG_LD)
+	    int argc, const word *argv, int pair)
 { list l = 0;
   Word top = NULL;
 
   if ( !ensureLocalSpace(sizeof(word)) )
-    return FALSE;
+    return false;
+
+  static_assertion(sizeof(*l) == 3*sizeof(word));
 
   switch( prolog_list_to_sort_list(in, remove_dups,
 				   argc, argv, pair,
-				   &l, &top PASS_LD) )
+				   &l, &top) )
   { case SORT_ERR:
-      return FALSE;
+      return false;
     case SORT_NIL:
       return PL_unify_nil(out);
     case SORT_NOSORT:
+      DEBUG(CHK_SECURE, checkStacks(NULL));
       return PL_unify(in, out);
     case SORT_SORT:
     default:
@@ -537,6 +564,7 @@ pl_nat_sort(term_t in, term_t out,
       l = nat_sort(l, remove_dups, order);
       put_sort_list(tmp, l);
       gTop = top;
+      DEBUG(CHK_SECURE, checkStacks(NULL));
 
       return PL_unify(out, tmp);
     }
@@ -549,8 +577,8 @@ PRED_IMPL("sort", 2, sort, PL_FA_ISO)
 { PRED_LD
 
   return pl_nat_sort(A1, A2,
-		     TRUE, SORT_ASC,
-		     0, NULL, FALSE PASS_LD);
+		     true, SORT_ASC,
+		     0, NULL, false);
 }
 
 
@@ -559,8 +587,8 @@ PRED_IMPL("msort", 2, msort, 0)
 { PRED_LD
 
   return pl_nat_sort(A1, A2,
-		     FALSE, SORT_ASC,
-		     0, NULL, FALSE PASS_LD);
+		     false, SORT_ASC,
+		     0, NULL, false);
 }
 
 
@@ -569,8 +597,8 @@ PRED_IMPL("keysort", 2, keysort, PL_FA_ISO)
 { PRED_LD
 
   return pl_nat_sort(A1, A2,
-		     FALSE, SORT_ASC,
-		     0, NULL, TRUE PASS_LD);
+		     false, SORT_ASC,
+		     0, NULL, true);
 }
 
 /** sort(+Key, +Order, +Random, -Sorted)
@@ -580,28 +608,29 @@ ECLiPSe compatible sort.
 
 #define FAST_ARGV 10
 
+#define get_key_arg_ex(t, k, zero_ok) LDFUNC(get_key_arg_ex, t, k, zero_ok)
 static int
-get_key_arg_ex(term_t t, word *k, int zero_ok ARG_LD)
+get_key_arg_ex(DECL_LD term_t t, word *k, int zero_ok)
 { Word p = valTermRef(t);
 
   deRef(p);
   if ( isTaggedInt(*p) )
-  { intptr_t v = valInt(*p);
+  { intptr_t v = (intptr_t)valInt(*p);
 
     if ( v > 0 )
     { *k = *p;
-      return TRUE;
+      return true;
     }
     if ( v == 0 )
     { *k = *p;
       if ( zero_ok )
-	return TRUE;
+	return true;
     }
   }
 
   if ( isAtom(*p) )
   { *k = *p;
-    return TRUE;
+    return true;
   }
 
   if ( isInteger(*p) )
@@ -611,7 +640,7 @@ get_key_arg_ex(term_t t, word *k, int zero_ok ARG_LD)
     if ( ar_sign_i(&n) <= 0 )
       PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_not_less_than_one, t);
 
-    return FALSE;
+    return false;
   }
 
   return -1;
@@ -624,14 +653,14 @@ typedef struct order_def
 } order_def;
 
 static const order_def order_defs[] =
-{ { ATOM_smaller,	SORT_ASC,  TRUE	 },
-  { ATOM_at_smaller,	SORT_ASC,  TRUE	 },
-  { ATOM_smaller_equal,	SORT_ASC,  FALSE },
-  { ATOM_at_smaller_eq,	SORT_ASC,  FALSE },
-  { ATOM_larger,	SORT_DESC, TRUE	 },
-  { ATOM_at_larger,	SORT_DESC, TRUE	 },
-  { ATOM_larger_equal,	SORT_DESC, FALSE },
-  { ATOM_at_larger_eq,	SORT_DESC, FALSE },
+{ { ATOM_smaller,	SORT_ASC,  true	 },
+  { ATOM_at_smaller,	SORT_ASC,  true	 },
+  { ATOM_smaller_equal,	SORT_ASC,  false },
+  { ATOM_at_smaller_eq,	SORT_ASC,  false },
+  { ATOM_larger,	SORT_DESC, true	 },
+  { ATOM_at_larger,	SORT_DESC, true	 },
+  { ATOM_larger_equal,	SORT_DESC, false },
+  { ATOM_at_larger_eq,	SORT_DESC, false },
   { 0 }
 };
 
@@ -646,9 +675,9 @@ PRED_IMPL("sort", 4, sort, 0)
   atom_t order_name;
   const order_def *od;
 
-  if ( (rc=get_key_arg_ex(A1, argv, TRUE PASS_LD)) == FALSE )
-    return FALSE;
-  if ( rc == TRUE )				/* Key is integer */
+  if ( (rc=get_key_arg_ex(A1, argv, true)) == false )
+    return false;
+  if ( rc == true )				/* Key is integer */
   { if ( argv[0] == consInt(0) )
     { argc = 0;
       argv = NULL;
@@ -668,8 +697,8 @@ PRED_IMPL("sort", 4, sort, 0)
 	    return PL_no_memory();
 	}
         for(argc=0; PL_get_list(tail, head, tail); argc++)
-	{ if ( get_key_arg_ex(head, &argv[argc], FALSE PASS_LD) != TRUE )
-	  { rc = FALSE;
+	{ if ( get_key_arg_ex(head, &argv[argc], false) != true )
+	  { rc = false;
 	    goto out;
 	  }
 	}
@@ -695,7 +724,7 @@ PRED_IMPL("sort", 4, sort, 0)
 
   rc = pl_nat_sort(A3, A4,
 		   od->remove_dups, od->order,
-		   argc, argv, FALSE PASS_LD);
+		   argc, argv, false);
 
 out:
   if ( argv && argv != tmp )
@@ -709,11 +738,11 @@ out:
 		 *******************************/
 
 BeginPredDefs(list)
-  PRED_DEF("is_list", 1, is_list, 0)
-  PRED_DEF("$length", 2, dlength, 0)
-  PRED_DEF("memberchk", 2, memberchk, 0)
-  PRED_DEF("sort", 2, sort, PL_FA_ISO)
-  PRED_DEF("msort", 2, msort, 0)
-  PRED_DEF("keysort", 2, keysort, PL_FA_ISO)
-  PRED_DEF("sort", 4, sort, 0)
+  PRED_DEF("is_list",	 1, is_list,   0)
+  PRED_DEF("$length",	 2, dlength,   0)
+  PRED_DEF("$memberchk", 3, memberchk, 0)
+  PRED_DEF("sort",	 2, sort,      PL_FA_ISO)
+  PRED_DEF("msort",	 2, msort,     0)
+  PRED_DEF("keysort",	 2, keysort,   PL_FA_ISO)
+  PRED_DEF("sort",	 4, sort,      0)
 EndPredDefs
